%author Veronica Martinez, The University of Queensland, AIBN
%19-March-2014

%B-DMFA program to estimate mass balance fluxes, the main outputs are: 
% to, the knots positions
% P, control points
% V2, estimated fluxes over time
% C2, fitted concentrations over time
% SSR, the variance-weighted sum of squared residuals 
% MinCI2, min rates CI
% MaxCI2, max rates CI 

clear all;clc;
tic
% load data cm ( measured concentrations), rm (measured rates),
% S (matrix with balanced metabolites), R (concentrations measurement matrix), 
%t (time points of measured data), stdev (standar desviation of data, concentrations)
%Rr (rates measurement matrix), stdevR (standar desviation of data, rates)
addpath ./fxns 

[t,tr,cm,rm,stdev,stdevR,R,Rr,S,metabM,namesR]=LoadData(1,'TemperatureShift.xlsx','b');
%run YeastExample 

to=[t(1) t(end)]';
to=[to(1) to(1) to' to(end) to(end)+0.01]; % with two extra knots at the extreme time points
K=null(S);
[P,co,SSR,~,~,p,pError]=CalculateValues(S,R,Rr,cm,rm,t,to,tr,stdev,stdevR,K);
SSRi=SSR;
% Start with the algoritm to locate the internal knots
% Estimate goodness-of-fit, with chi-squared considering 95% CI
n= length(t)*(size(cm,2))+length(tr)*(size(rm,2)); % fitted measurements
pa=length(p); % estimated parameters
np=n-pa; % redundant data
x=chi2inv(1-0.025,np); % 95% CI

Step=0;
divisions=1;
while SSR>x && np > size(P,1)+1
    toint=to(3:end-2);
    SSRcheck=zeros(divisions,1);
    for i=1:divisions
        if i==1
            mini=1;
            maxi=max(find(t<=toint(2)));
        elseif i==length(divisions)
            mini=min(find(t>=toint(end-1)));
            maxi=legth(t);
        else
            mini=min(find(t>=toint(i)));
            maxi=max(find(t<=toint(i+1)));
        end
        SSRcheck(i)=EstimateSSR(to,t,tr,cm,rm,R,Rr,P,K,co,stdev,stdevR,mini,maxi,pError);
    end
    workplace=find(SSRcheck==max(SSRcheck));
    extraP=toint(workplace)+(1/2)*(toint(workplace+1)-toint(workplace));
    to=[to(1:workplace+2) extraP to(workplace+3:end)];
    [~,~,SSR,~,~,p,pError]=CalculateValues(S,R,Rr,cm,rm,t,to,tr,stdev,stdevR,K);
    pa=length(p)+length(to)-6; % estimated parameters
    np=n-pa; % redundant data
    x=chi2inv(1-0.025,np); %95% CI
    % move the position of extra knot
    [step,SSRtot,SSR,to,P,co,~,~,pError]=MovePoint2(find(to==extraP),x,S,R,Rr,cm,rm,t,to,tr,stdev,stdevR,100,SSR,K);
    %move all internal knots 
    if Step>0
        SSRpre=SSR+1;
        while SSR<SSRpre
            SSRpre=SSR;
            for i=1:length(to)-6
                [~,~,SSR,to,P,co,~,~,pError]=MovePoint2(i+3,x,S,R,Rr,cm,rm,t,to,tr,stdev,stdevR,100,SSR,K);
            end
        end
    end
    Step=Step+1;
    divisions=divisions+1;
    fprintf('\n %1.0f\n',Step);
end

%The final result (fluxes)
[P,co,SSR,V,C,p,pError]=CalculateValues(S,R,Rr,cm,rm,t,to,tr,stdev,stdevR,K);
Error=[];
for i=1:length(t)
    ti=t(i);
    [v,Stv]=calculatev(K,P,ti,to,pError);
    Error=[Error; Stv'];
end
% determine 95% confidence interval
MinCI=V-(2*Error);
MaxCI=V+(2*Error);

t2=t(1):0.1:t(end);
V2=[];
C2=[];
Error2=[];
Me=IntN3(to);
for j=1:length(t2)
    tj=t2(j);
    N=Nblendfxn(tj,to);
    IN4=Nblendfxn4(tj,to);
    v=K*P*N';
    c=co+(R*K*P*Me*IN4');
    Stdv2=pError*(N.^2)';
    V2=[V2; v'];
    C2=[C2; c'];
    Error2=[Error2; Stdv2'];
end
% determine 95% CI
MinCI2=V2-(2*Error2);
MaxCI2=V2+(2*Error2);

toc
%run PlotYeast

